@@ -7,7 +7,7 @@
 # Contact: Kyle Lahnakoski (kyle@lahnakoski.com)
 #
 
-from mo_dots import Data, Null, coalesce, is_data, is_list, wrap
+from mo_dots import Data, Null, coalesce, is_data, is_list, to_data
 from mo_future import PY2, is_text, text, unichr, urlparse, is_binary
 from mo_logs import Log
 
@@ -44,14 +44,14 @@ class URL(object):
                 scheme, suffix = value.split("//", 2)
                 self.scheme = scheme.rstrip(":")
                 parse(self, suffix, 0, 1)
-                self.query = wrap(url_param2value(self.query))
+                self.query = to_data(url_param2value(self.query))
             else:
                 output = urlparse(value)
                 self.scheme = output.scheme
                 self.port = coalesce(port, output.port)
                 self.host = output.netloc.split(":")[0]
                 self.path = coalesce(path, output.path)
-                self.query = coalesce(query, wrap(url_param2value(output.query)))
+                self.query = coalesce(query, to_data(url_param2value(output.query)))
                 self.fragment = coalesce(fragment, output.fragment)
         except Exception as e:
             Log.error(u"problem parsing {{value}} to URL", value=value, cause=e)
@@ -73,6 +73,13 @@ class URL(object):
         output.path = output.path.rstrip('/') + "/" + other.lstrip('/')
         return output
 
+    def __add__(self, other):
+        if not is_data(other):
+            Log.error("can only add data for query parameters")
+        output = self.__copy__()
+        output.query += other
+        return output
+
     def __unicode__(self):
         return self.__str__().decode('utf8')  # ASSUME chr<128 ARE VALID UNICODE
 
@@ -92,6 +99,12 @@ class URL(object):
     def __data__(self):
         return str(self)
 
+    def __hash__(self):
+        return hash(str(self))
+
+    def __eq__(self, other):
+        return text(self) == text(other)
+
     def __str__(self):
         url = ""
         if self.host:
@@ -217,11 +230,11 @@ def value2url_param(value):
     if is_data(value):
         from mo_json import value2json
 
-        value_ = wrap(value)
-        output = "&".join([
+        value_ = to_data(value)
+        output = "&".join(
             value2url_param(k) + "=" + (value2url_param(v) if is_text(v) else value2url_param(value2json(v)))
-            for k, v in value_.leaves()
-            ])
+            for k, v in sorted(value_.leaves(), key=lambda p: p[0])
+        )
     elif is_text(value):
         output = "".join(_map2url[c] for c in value.encode('utf8'))
     elif is_binary(value):
